# Copyright (c) 2025 Orange. All rights reserved.
# This software is distributed under the BSD 3-Clause-clear License, the text of which is available
# at https://spdx.org/licenses/BSD-3-Clause-Clear.html or see the "LICENSE" file for more details.

with section("format"):
    line_width = 120
    tab_size = 2
    line_ending = "auto"

with section("markup"):
    bullet_char = "-"
